/** cube2.cpp
*
*   Demonstrate OpenGL functions
*
*      Add light to scene
*
**/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <math.h>
#include <windows.h>
#include "GLapplic.h"

#define OBJ_CUBE  1

class MyApp: public CAppWindow
{
	public:
		MyApp(char *name);
		void PrepareScene();
		void DrawScene();
};

MyApp::MyApp(char *name): CAppWindow(name)
{
	Width  = 480 ;
	Height = 360 ;
}

void MyApp::PrepareScene()
{
	glNewList(OBJ_CUBE, GL_COMPILE) ; 

	//
	// Draw the six faces of the cube.
	//
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);            
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, -1.0f);
	glVertex3f (1.0f, 1.0f, -1.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (-1.0f, 0.0f, 0.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (1.0f, 0.0f, 0.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 1.0f, 0.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);            
	glVertex3f (1.0f, 1.0f, 1.0f);             
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, -1.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, -1.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();

	glEndList() ;			   
}

void MyApp::DrawScene()
{
	GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	GLfloat glfMaterialColor[] = { 0.0f, 0.0f, 1.0f, 1.0f };

	//
	// Enable depth testing and clear the color and depth buffers.
	//
	glEnable (GL_DEPTH_TEST);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//
	// Add a light to the scene.
	//
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);

	//
	// Define the modelview transformation.
	//
	glTranslatef (0.0f, 0.0f, -8.0f);
	glRotatef (30.0f, 1.0f, 0.0f, 0.0f);
	glRotatef (70.0f, 0.0f, 1.0f, 0.0f);

	//
	// Define the reflective properties of the cube's faces.
	//
	glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, glfMaterialColor);


	// Draw the cube
	glCallList(OBJ_CUBE) ;

	//
	// Flush the drawing pipeline.
	//
	glFlush ();
}

int Main()
{
	return MyApp("cube2").Run();
}
