/** cube4.cpp
*
*   Demonstrate OpenGL functions
*
*      Multiple instances of display list.
*
**/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <math.h>
#include <windows.h>
#include "GLapplic.h"

#define OBJ_CUBE  1

class MyApp: public CAppWindow
{
	public:
		MyApp(char *name);
		int nAngle;
		void SetupWindow();
		void CleanupWindow();
		void WmTimer();
		void PrepareScene();
		void DrawScene();
};

MyApp::MyApp(char *name): CAppWindow(name)
{
	Width  = 480 ;
	Height = 360 ;
	nAngle = 70;
}

void MyApp::SetupWindow()
{
	SetTimer (hWnd, 1, 50, NULL);
}

void MyApp::CleanupWindow()
{
	KillTimer(hWnd, 1);
}

void MyApp::WmTimer()
{
	nAngle += 2;
	if (nAngle >= 90)
		nAngle = 0;
	InvalidateRect (hWnd, NULL, FALSE);         
}

void InitializeRC (void)
{
	GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };

	//
	// Enable depth testing and backface culling.
	//
	glEnable (GL_DEPTH_TEST);
	glEnable (GL_CULL_FACE);

	//
	// Add a light to the scene.
	//
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
}

void MakeCube()
{
	glNewList(OBJ_CUBE, GL_COMPILE) ; 

	//
	// Draw the six faces of the cube.
	//
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);            
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, -1.0f);
	glVertex3f (1.0f, 1.0f, -1.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (-1.0f, 0.0f, 0.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (1.0f, 0.0f, 0.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 1.0f, 0.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);            
	glVertex3f (1.0f, 1.0f, 1.0f);             
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	glBegin (GL_POLYGON);
	glNormal3f (0.0f, -1.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, -1.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();

	glEndList() ;			   
}

void MyApp::PrepareScene()
{
	InitializeRC();
	MakeCube();
}



void MyApp::DrawScene()
{
	GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	GLfloat glfMaterialColor[] = { 0.0f, 0.0f, 1.0f, 1.0f };

	//
	// Enable depth testing and clear the color and depth buffers.
	//
	glEnable (GL_DEPTH_TEST);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//
	// Add a light to the scene.
	//
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);

	//
	// Define the modelview transformation.
	//
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glTranslatef (0.0f, 0.0f, -32.0f);
	glRotatef (30.0f, 1.0f, 0.0f, 0.0f);
	glRotatef ((GLfloat) nAngle, 0.0f, 1.0f, 0.0f);

	//
	// Define the reflective properties of the cube's faces.
	//
	glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, glfMaterialColor);


	//
	// Draw a three-dimensional array of cubes.
	//
	int i, j, k;
	for (i=-3; i<=3; i+=3) {
		for (j=-3; j<=3; j+=3) {
			for (k=-3; k<=3; k+=3) {
				glPushMatrix ();
				glTranslatef ((GLfloat) i, (GLfloat) j, (GLfloat) k);
				glCallList (OBJ_CUBE);
				glPopMatrix ();
			}
		}
	}

	//
	// Flush the drawing pipeline.
	//
	glFlush ();
}

int Main()
{
	return MyApp("cube4").Run();
}
