/** cube3.cpp
*
*   Demonstrate OpenGL functions
*
*      Add animation
*
**/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <math.h>
#include <windows.h>
#include "GLapplic.h"

#define OBJ_CUBE  1

class MyApp: public CAppWindow
{
	public:
		MyApp(char *name);
		int nAngle;
		void SetupWindow();
		void CleanupWindow();
		void WmTimer();
		void PrepareScene();
		void DrawScene();
};

MyApp::MyApp(char *name): CAppWindow(name)
{
	Width  = 480 ;
	Height = 360 ;
	nAngle = 70;
}

void MyApp::SetupWindow()
{
	SetTimer (hWnd, 1, 50, NULL);
}

void MyApp::CleanupWindow()
{
	KillTimer(hWnd, 1);
}

void MyApp::WmTimer()
{
	nAngle += 2;
	if (nAngle >= 90)
		nAngle = 0;
	InvalidateRect (hWnd, NULL, FALSE);         
}


void polygon(int a, int b, int c, int d)
{
	const GLfloat s = 1.0f;

	const GLfloat vertices[8][3] = {
		{-s,-s,-s},{s,-s,-s},{s,s,-s},{-s,s,-s},
		{-s,-s,s},{s,-s,s},{s,s,s},{-s,s,s}
	};
	const GLfloat normals[8][3] = {
		{-s,-s,-s},{s,-s,-s},{s,s,-s},{-s,s,-s},
		{-s,-s,s},{s,-s,s},{s,s,s},{-s,s,s}
	};
	const GLfloat colors[8][3] = {
		{0.0f,0.0f,0.0f},{1.0f,0.0f,0.0f},{1.0f,1.0f,0.0f},{0.0f,1.0f,0.0f},
		{0.0f,0.0f,1.0f},{1.0f,0.0f,1.0f},{1.0f,1.0f,1.0f},{0.0f,1.0f,1.0f}
	};
	int i, k;
	glBegin(GL_POLYGON);
	for (i=0; i<4; i++) {
		k = (i==0? a: (i==1? b: (i==2? c: d)));
		glColor3fv(colors[k]);
		glNormal3fv(normals[k]);
		glVertex3fv(vertices[k]);
	}
	glEnd();
}

void MakeCube()
{
	glNewList(OBJ_CUBE, GL_COMPILE) ; 

	polygon(0,3,2,1);
	polygon(2,3,7,6);
	polygon(0,4,7,3);
	polygon(1,2,6,5);
	polygon(4,5,6,7);
	polygon(0,1,5,4);

	glEndList() ;			   
}

void MyApp::PrepareScene()
{
	MakeCube();
}



void MyApp::DrawScene()
{

	//
	// Enable depth testing and clear the color and depth buffers.
	//
	glEnable (GL_DEPTH_TEST);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable (GL_CULL_FACE);


	//
	// Define the modelview transformation.
	//
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glTranslatef (0.0f, 0.0f, -8.0f);
	glRotatef (30.0f, 1.0f, 0.0f, 0.0f);
	glRotatef ((GLfloat) nAngle, 0.0f, 1.0f, 0.0f);



	// Draw the cube
	glCallList(OBJ_CUBE) ;

	//
	// Flush the drawing pipeline.
	//
	glFlush ();
}

int Main()
{
	return MyApp("cube5").Run();
}
